// $Id: RandGaussT.h,v 1.5 2002/04/12 15:02:44 evc Exp $
// -*- C++ -*-
//
// -----------------------------------------------------------------------
//                             HEP Random
//                          --- RandGaussT ---
//                          class header file
// -----------------------------------------------------------------------

// Class defining methods RandGaussT, which is derived from RandGauss.
// The user interface is identical.  
// RandGaussT ---
//   Always uses exactly one flat random from the engine
//   Is stateless so that saveEngineStatus works even if invoked knowing
//   only that the distribution is a HepRandom.  (RandGauss is not stateless.)
//   Is accurate to more than 12 decimal places in all cases, and more so 
//   near the end points.
//   Runs about as fast as RandGauss.
// RandGaussT utilizes HepStat::flatToGaussian(r), so its effective footprint
// (for the tables used) is almost 30K bytes.

// =======================================================================
// M. Fischler		Created 2/2/00
//
// =======================================================================

#ifndef RandGaussT_h
#define RandGaussT_h 1

#include "CLHEP/Random/RandGauss.h"
#include "CLHEP/Random/Stat.h"

/**
 * @author
 * @ingroup random
 */
class RandGaussT : public RandGauss {

public:

  inline RandGaussT ( HepRandomEngine& anEngine, double mean=0.0,
                                                double stdDev=1.0 );
  inline RandGaussT ( HepRandomEngine* anEngine, double mean=0.0,
                                                double stdDev=1.0 );
  // These constructors should be used to instantiate a RandGaussT
  // distribution object defining a local engine for it.
  // The static generator will be skipped using the non-static methods
  // defined below.
  // If the engine is passed by pointer the corresponding engine object
  // will be deleted by the RandGaussT destructor.
  // If the engine is passed by reference the corresponding engine object
  // will not be deleted by the RandGaussT destructor.

  // Destructor
  virtual ~RandGaussT();

  //
  // Methods to generate Gaussian-distributed random deviates:
  //
  //   If a fast good engine takes 1 usec, RandGauss::fire() adds 1 usec;
  //   RandGaussT::fire() similarly adds 1 usec.
  //

  // Static methods to shoot random values using the static generator

  static  inline double shoot();

  static  inline double shoot( double mean, double stdDev );

  static  void shootArray ( const int size, double* vect,
                            double mean=0.0, double stdDev=1.0 );

  //  Static methods to shoot random values using a given engine
  //  by-passing the static generator.

  static  inline double shoot( HepRandomEngine* anotherEngine );

  static  inline double shoot( HepRandomEngine* anotherEngine, 
                                  double mean, double stdDev );


  static  void shootArray ( HepRandomEngine* anotherEngine, 
			    const int size,
                            double* vect, double mean=0.0,
                            double stdDev=1.0 );

  //  Instance methods using the localEngine to instead of the static 
  //  generator, and the default mean and stdDev established at construction

  inline double fire();

  inline double fire ( double mean, double stdDev );
  
  void fireArray  ( const int size, double* vect);
  void fireArray  ( const int size, double* vect,
                    double mean, double stdDev );

  virtual double operator()();
  virtual double operator()( double mean, double stdDev );

protected:

private:

  // Private copy constructor. Defining it here disallows use.
  RandGaussT(const RandGaussT& d);

  // All the engine info, and the default mean and sigma, are in the RandGauss
  // base class.

};

#include "CLHEP/Random/RandGaussT.icc"

#endif
